import requests
import threading
import urllib.parse


class APIBase:
	def __init__(self, threaded):
		if threaded:
			# Create and start a new thread to send the request.

			self._thread: threading.Thread = threading.Thread(target=self._start)

			self._thread.start()

		else:
			# Single-threaded - We call the start method on the main thread
			self._start()

	def _join_thread(self):
		""" Handles the extra thread by joining it onto the main thread. """

		# If a thread object exists and it is still active, wait for it to finish.
		if hasattr(self, "_thread") and self._thread.is_alive():
			self._thread.join()

	def _start(self) -> None:
		raise NotImplementedError()

	@staticmethod
	def send_request(url: str) -> requests.Response:
		"""
		Send a request to the URL provided

		:param str url: The URL which we are sending a GET request to.
		:raise: Will raise exceptions from the requests module
		:return: The response object or None
		"""

		headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:76.0) Gecko/20100101 Firefox/76.0'}

		return requests.get(url, stream=True, timeout=5, headers=headers)

	@staticmethod
	def send_request_image(url: str) -> requests.Response:
		"""
		Send a request to the URL provided

		:param str url: The URL which we are sending a GET request to.
		:raise: Will raise exceptions from the requests module
		:return: The response object or None
		"""
		domain = urllib.parse.urlparse(url).netloc

		header = {
			'Accept': 'image/png,image/svg+xml,image/*;q=0.8,video/*;q=0.8,*/*;q=0.5',
			'Accept-Encoding': 'gzip, deflate, br',
			'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/13.1.2 Safari/605.1.15',
			'Host': domain, 'Accept-Language': 'en-ca', 'Referer': 'https://manganelo.com/',
			'Connection': 'keep-alive'
		}

		r = requests.get(url, stream=True, timeout=5, headers=header)

		return r



import os
import shutil
import tempfile

from bs4 import BeautifulSoup

from PIL import Image

from reportlab.pdfgen import canvas

from manganelo.rewrite import utils, siterequests


class ChapterDownloader:
	def __init__(self, url):
		self._url = url

	def download(self, path):
		path = utils.validate_path(path)

		r = siterequests.get(self._url)

		soup = BeautifulSoup(r.content, "html.parser")

		urls = self._get_chapter_image_urls(soup)

		with tempfile.TemporaryDirectory() as dir_:
			images = self._download_images(dir_, urls)

			self._create_pdf(path, images)

		return path

	@staticmethod
	def _get_chapter_image_urls(soup):

		def valid(url: str):
			return url.endswith((".png", ".jpg"))

		return [url for url in map(lambda ele: ele["src"], soup.find_all("img")) if valid(url)]

	@staticmethod
	def _download_images(dir_, urls: list):
		images = []

		for i, url in enumerate(urls):
			image = siterequests.dl_image(url)

			if image is not None:
				ext = url.split(".")[-1]

				with open(os.path.join(dir_, f"{i}.{ext}"), "wb") as fh:
					image.raw.decode_content = True

					# noinspection PyBroadException
					try:
						shutil.copyfileobj(image.raw, fh)

					except:
						continue

					images.append(fh.name)

		return images

	@staticmethod
	def _create_pdf(path, images: list):

		pdf = canvas.Canvas(path)

		for image in images:

			# noinspection PyBroadException
			try:
				with Image.open(image) as img:
					w, h = img.size

			except:
				continue

			pdf.setPageSize((w, h))  # Set the page dimensions to the image dimensions

			pdf.drawImage(image, x=0, y=0)  # Insert the image onto the current page

			pdf.showPage()  # Create a new page ready for the next image

		pdf.save()



import requests
import urllib.parse


_ROOT_URL  = "http://manganelo.com/"


def get(url):
	return requests.get(url, stream=True, timeout=10)


def search(title):
	return get(f"{_ROOT_URL}search/story/{title}")


def dl_image(url):
	header = {
		'Accept': 'image/png,image/svg+xml,image/*;q=0.8,video/*;q=0.8,*/*;q=0.5',
		'Accept-Encoding': 'gzip, deflate, br',
		'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/13.1.2 Safari/605.1.15',
		'Host': urllib.parse.urlparse(url).netloc, 'Accept-Language': 'en-ca', 'Referer': _ROOT_URL,
		'Connection': 'keep-alive'
	}

	return requests.get(url, stream=True, timeout=10, headers=header)



from .searchmanga import SearchManga
from .mangainfo import MangaInfo
from .downloadchapter import DownloadChapter
from .chapterinfo import ChapterInfo

__ALL__ = (
    "SearchManga",
    "MangaInfo",
    "DownloadChapter",
    "ChapterInfo"
)




import locale

from datetime import datetime


def parse_date(s: str, _format: str):

	try:
		# Standardize locale to match foreign language
		locale.setlocale(locale.LC_ALL, "en_US.UTF8")

		date = datetime.strptime(s, _format)

	finally:
		# Rese-set the locale
		locale.setlocale(locale.LC_ALL, '')

	return date



import ast
import string
import typing

import functools as ft

from bs4 import BeautifulSoup

from manganelo.rewrite import utils, siterequests

from manganelo.rewrite.mangapage import MangaPageGetter, Chapter


class SearchResult:
	def __init__(self, soup):
		self._soup = soup

	@ft.cached_property
	def title(self): return self._soup.find(class_="item-img").get("title", None)

	@ft.cached_property
	def url(self): return self._soup.find(class_="item-img").get("href", None)

	@ft.cached_property
	def icon_url(self): return self._soup.find("img", class_="img-loading").get("src", None)

	@ft.cached_property
	def updated(self):
		s = self._soup.find("span", class_="text-nowrap item-time").text

		return utils.parse_date(s, "Updated : %b %d,%Y - %H:%M")

	@ft.cached_property
	def authors(self):
		return [e.strip() for e in self._soup.find("span", class_="text-nowrap item-author").text.split(",")]

	@ft.cached_property
	def views(self):
		s = self._soup.find_all("span", class_="text-nowrap item-time")[-1].text

		return ast.literal_eval(s.replace("View : ", "").replace(",", ""))

	@ft.cached_property
	def rating(self): return ast.literal_eval(self._soup.find("em", class_="item-rate").text)

	@ft.lru_cache()
	def chapter_list(self) -> typing.List[Chapter]:
		return MangaPageGetter(self.url).get().chapter_list()


class _MangaSearch:
	def __init__(self, title: str):
		self._raw_title = title

	def get(self):
		r = self._send_request()

		return self._extract_response(r)

	def _send_request(self):
		return siterequests.search(self._validate_title(self._raw_title))

	def _extract_response(self, resp) -> list:
		soup = BeautifulSoup(resp.content, "html.parser")

		return [SearchResult(ele) for ele in soup.find_all(class_="search-story-item")]

	@staticmethod
	def _validate_title(title: str) -> str:
		allowed_characters: str = string.ascii_letters + string.digits + "_"

		return "".join([char.lower() for char in title.replace(" ", "_") if char in allowed_characters])



import os
import locale

import datetime as dt


def parse_date(s: str, _format: str):

	try:
		locale.setlocale(locale.LC_ALL, "en_US.UTF8")

		return dt.datetime.strptime(s, _format)

	finally:
		locale.setlocale(locale.LC_ALL, '')


def validate_path(path: str):
	dir_, file = os.path.split(path)

	for char in ("\\", "/", ":", "*", "?", "<", ">", "|"):
		file = file.replace(char, "")

	return os.path.join(dir_, file)


import dataclasses

import functools as ft

from typing import List

from bs4 import BeautifulSoup

from manganelo.api.apibase import APIBase


@dataclasses.dataclass
class ChapterInformation:
	title: str
	url: str
	image_urls: List[str]


class ChapterInfo(APIBase):
	def __init__(self, src_url: str, *, threaded: bool = False):

		self._src_url = src_url

		self._soup = None

		super(ChapterInfo, self).__init__(threaded=threaded)

	@classmethod
	def from_soup(cls, soup: BeautifulSoup):
		return ChapterInformation(
			title=cls._get_chapter_title(soup),
			url="N/A",
			image_urls=cls._get_image_urls(soup)
		)

	@ft.cached_property
	def results(self):
		self._join_thread()

		return ChapterInformation(
			title=self._get_chapter_title(self._soup),
			url=self._src_url,
			image_urls=self._get_image_urls(self._soup)
		)

	def _start(self) -> None:
		response = self.send_request(self._src_url)

		self._soup: BeautifulSoup = BeautifulSoup(response.content, "html.parser")

	@classmethod
	def _get_image_urls(cls, soup) -> List[str]:
		"""
		Return all the image URLS inside the soup object.

		:return: We return a list of image URLS
		"""

		def valid(url: str):
			return url.endswith((".png", ".jpg"))

		image_soup = soup.find_all("img")

		images = [url for url in map(lambda ele: ele["src"], image_soup) if valid(url)]

		return images

	@classmethod
	def _get_chapter_title(cls, soup: BeautifulSoup) -> str:
		""" Return the title of the chapter. """

		title = soup.find(class_="panel-chapter-info-top")

		return title.find("h1").text if title else None



import os
import shutil
import tempfile
import typing
import dataclasses

import functools as ft

from reportlab.pdfgen import canvas
from bs4 import BeautifulSoup
from PIL import Image

from manganelo.api.apibase import APIBase
from manganelo.api.chapterinfo import ChapterInfo


@dataclasses.dataclass
class ChapterStatus:
	title: str
	saved_ok: bool
	percent_saved: float
	path: str


class DownloadChapter(APIBase):
	def __init__(self, src_url: str, dst_path: str, *, threaded: bool = False):
		"""
		Object constructor

		:param src_url: The  chapter which we will be downloading
		:param dst_path: The path where the chapter will be saved after completion
		:param threaded: Whether the bulk of the work will be done on a seperate thread or the main thread
		"""

		self._src_url = src_url
		self._dst_path = self._update_destination_path(dst_path)
		self._title = None

		self._saved = False
		self._percent_saved = 0

		super(DownloadChapter, self).__init__(threaded)

	@ft.cached_property
	def results(self):
		"""
		Returns the status of the download.

		:return ChapterStatus: The status of the chapter
		"""

		self._join_thread()

		return ChapterStatus(
			saved_ok=self._saved,
			percent_saved=self._percent_saved,
			path=self._dst_path,
			title=self._title
		)

	def _start(self):
		""" The main function...Where the magic happens. """

		r = self.send_request(self._src_url)

		soup = BeautifulSoup(r.content, "html.parser")

		chap = ChapterInfo.from_soup(soup)

		self._title = chap.title

		with tempfile.TemporaryDirectory() as temp_dir:
			image_paths = self._download_images(chap.image_urls, temp_dir)

			num_pages = self._create_pdf(image_paths)

			self._percent_saved = num_pages / len(chap.image_urls)

	def _download_images(self, image_urls: typing.List[str], save_dir: str) -> typing.List[str]:
		"""
		Download images from a sequence of URLS into a directory.

		:param image_urls: List of URLS which we will attempt to download here
		:param save_dir: The directory where the downloaded images will be saved
		:return list: List of paths where the downloaded images are stored
		"""

		image_paths = []

		for i, url in enumerate(image_urls):
			image = self.send_request_image(url)

			image_ext = url.split(".")[-1]

			image_dst_path = os.path.join(save_dir, f"{i}.{image_ext}")

			if image is not None:
				with open(image_dst_path, "wb") as fh:

					# Magic boolean which makes it work
					image.raw.decode_content = True

					# noinspection PyBroadException

					# Attempt to download the image from the URL
					try:
						shutil.copyfileobj(image.raw, fh)

					# We should reduce the scope
					except Exception:
						pass

					# We downloaded the image without any errors
					else:
						image_paths.append(image_dst_path)

		return image_paths

	def _update_destination_path(self, path):
		""" Remove the illegal characters from the file path. """

		dir_, file = os.path.split(path)

		for char in ("\\", "/", ":", "*", "?", "<", ">", "|"):
			file = file.replace(char, " ")

		return os.path.join(dir_, file)

	def _create_pdf(self, images: typing.List[str]) -> int:
		"""

		:param images: List of image paths which we will attempt to convert into a PDF
		:return int: The number of pages in the PDF
		"""

		pdf = canvas.Canvas(self._dst_path)

		num_pages = 0

		for image in images:
			try:
				with Image.open(image) as img:
					w, h = img.size

			except (OSError, UnboundLocalError):
				continue

			# Set the page dimensions to the image dimensions
			pdf.setPageSize((w, h))

			try:
				# Insert the image onto the current page
				pdf.drawImage(image, x=0, y=0)

			except OSError:
				continue

			# Create a new page ready for the next image
			pdf.showPage()

			num_pages += 1

		if num_pages > 0:
			dirs = os.path.dirname(self._dst_path)

			# Create the path if it doesn't exist already
			if dirs:
				os.makedirs(dirs, exist_ok=True)

			try:
				pdf.save()
			except FileNotFoundError:
				pass

			else:
				self._saved = True

		return num_pages



from setuptools import setup, find_packages


def read_file(file):
    with open(file, "r") as fh:
        return fh.read()


setup(
    name="manganelo",
    packages=find_packages(),
    version="1.18.2",
    license="MIT",

    description="Unofficial API for the Manganelo website.",
    long_description=read_file("README.md"),
    long_description_content_type="text/markdown",

    author="Joshua Nixon",
    author_email="joshuanixonofficial@gmail.com",

    url="https://github.com/nixonjoshua98/manganelo",

    download_url="https://github.com/nixonjoshua98/manganelo/archive/refs/tags/v1.18.2.tar.gz",
  
    keywords=["manga", "manganelo", "scrapper", "web", "mangakakalot", "thread", "comic", "manhwa", "manganato"],
  
    install_requires=[
        "bs4",
        "requests",
        "reportlab",
        "pillow"
    ],

    classifiers=[
      "Programming Language :: Python :: 3",
      "License :: OSI Approved :: MIT License",
      "Operating System :: OS Independent",
      "Development Status :: 5 - Production/Stable",
      "Intended Audience :: Developers",
      "Topic :: Software Development :: Build Tools",
    ],

    python_requires='>=3.7'
)



import dataclasses
import typing
import ast
import re

import functools as ft

from datetime import datetime
from bs4 import BeautifulSoup

from manganelo import utils

from manganelo.api.apibase import APIBase


@dataclasses.dataclass(frozen=True)
class MangaData:
    url: str
    title: str
    authors: list
    status: str
    genres: list
    alternative_titles: list
    chapters: list
    last_updated: datetime
    views: int
    icon: str
    description: str


@dataclasses.dataclass(frozen=True)
class MangaChapter:
    url: str
    title: str
    num: float


class MangaInfo(APIBase):
    def __init__(self, src_url: str, *, threaded: bool = False):
        """
        Constrctor for the object. We send the request here.

        :param str src_url: The URL which we will send a request to
        :param bool threaded: Determines if we want to send the request on the main thread or spawn a new thread.
        """

        self._src_url = src_url

        self._soup = None

        super(MangaInfo, self).__init__(threaded)

    def _start(self) -> None:
        """ Send the request and create the soup object """

        response = self.send_request(self._src_url)

        self._soup: BeautifulSoup = BeautifulSoup(response.content, "html.parser")

    @ft.cached_property
    def results(self) -> MangaData:
        """ Performs the soup extraction and returns an object """

        self._join_thread()

        table = self._parse_table()

        table.update(self._parse_extended_table())

        return MangaData(
            url=self._src_url,
            title=self._get_title(),
            status=table.get("status", None),
            authors=table.get("author", []),
            genres=table.get("genres", []),
            alternative_titles=table.get("alternative", []),
            chapters=self._get_chapter_list(),
            last_updated=table.get("updated", None),
            views=table.get("views", 0),
            icon=self._get_icon(),
            description=self._get_description()
        )

    def _get_title(self) -> typing.Union[str, None]:
        """ Return the title present on the page """

        story_info_right = self._soup.find(class_="story-info-right")

        return getattr(story_info_right.find("h1"), "text", None)

    def _get_icon(self) -> str:
        info_panel = self._soup.find(class_="panel-story-info")
        info_left = info_panel.find(class_="story-info-left")
        image = info_left.find(class_="info-image")

        return image.find("img").get("src", None)

    def _get_description(self) -> str:
        info_panel = self._soup.find(class_="panel-story-info")
        description = info_panel.find(class_="panel-story-info-description")

        return description.text.replace("Description :", "").strip()

    def _get_chapter_list(self) -> typing.List[MangaChapter]:
        """
        Extract the chapter list from the website

        :return list: Return a list of chapters which each contain information about the chapter
        """

        panels = self._soup.find(class_="panel-story-chapter-list")

        ls = []

        for i, ele in enumerate(reversed(panels.find_all(class_="a-h"))):
            if ele is not None:
                url = ele.find("a")["href"]
                text = ele.find("a").text

                num = ast.literal_eval(re.split("-|_", url.split("chapter")[-1])[-1])

                c = MangaChapter(url=url, num=num, title=text)

                ls.append(c)

        return ls

    def _parse_table(self) -> dict:
        """
        Parse the main table which contains the key information

        return dict: A dict of values taken from the page
        """

        table_section = self._soup.find(class_="variations-tableInfo")

        data = {}

        for row in table_section.find_all("tr"):
            label = row.find(class_="table-label")
            value = row.find(class_="table-value")

            # Eg. info-author -> author
            key = label.find("i").get("class")[0].split("-")[-1]
            val = None

            if key == "alternative":
                val = [ele.strip() for ele in value.text.split(";")]

            elif key == "author":
                val = [ele.text for ele in value.find_all("a")]

            elif key == "status":
                val = value.text.strip()

            elif key == "genres":
                val = [ele.text for ele in value.find_all("a")]

            data[key] = val

        return data

    def _parse_extended_table(self) -> dict:
        """
        Extract information from the extended table

        :return dict: Dict containing information taken from the extended table
        """

        right_extend = self._soup.find(class_="story-info-right-extent")

        rows = [ele.text for ele in right_extend.find_all("span", class_="stre-value") if ele.text.strip()]

        updated, views, *_ = rows

        updated = utils.parse_date(updated, "%b %d,%Y - %H:%M %p")

        views = int(views.replace(",", ""))

        return {"updated": updated, "views": views}





def search(*, title: str):
	from manganelo.rewrite.mangasearch import _MangaSearch

	return _MangaSearch(title).get()


def manga_page(*, url: str):
	from manganelo.rewrite.mangapage import MangaPageGetter

	return MangaPageGetter(url).get()


def chapter_list(*, url: str):
	return manga_page(url=url).chapter_list()


def download(*, url: str, path: str):
	from manganelo.rewrite.chapterdownloader import ChapterDownloader

	return ChapterDownloader(url).download(path)



import string
import dataclasses

import functools as ft

from bs4 import BeautifulSoup

from manganelo.api.apibase import APIBase


@dataclasses.dataclass(frozen=True)
class MangaSearchResult:
	title: str
	url: str


class SearchManga(APIBase):
	def __init__(self, query: str, *, threaded: bool = False) -> None:
		"""
		Constrctor for the object. We send the request here.

		:param str query: Query string which we will use as part of the URL which we will generate.
		:param bool threaded: Determines if we want to send the request on the main thread or spawn a new thread.
		"""

		self.url = None

		self._query = query
		self._response = None

		super(SearchManga, self).__init__(threaded)

	def _start(self) -> None:
		"""
		We generate the URL here and send the request.

		:raise: Exceptions from the requests module can be raised
		"""

		# Generate the URL, which includes removing 'illegal' characters
		self.url = self._generate_url(self._query)

		self._response = self.send_request(self.url)

	@ft.cached_property
	def results(self) -> list:
		"""
		Extract the results from the request we sent earlier.
		[Threaded] We join the thread, which means that we wait for the request to finish.

		:return Generator: Return a generator of the results
		"""

		self._join_thread()

		ls = []

		# Entire page soup
		soup = BeautifulSoup(self._response.content, "html.parser")

		# List of the search results
		results = soup.find_all(class_="search-story-item")

		# Iterate over the results soup and extract the information we want
		for i, ele in enumerate(results):
			manga = ele.find(class_="item-img")

			title = manga.get("title", None)  # Manga title
			link = manga.get("href", None)  # Link to the manga 'homepage'

			ls.append(MangaSearchResult(title=title, url=link))

		return ls

	@staticmethod
	def _generate_url(query: str) -> str:
		"""
		Generate the URL we send the request to, we remove all 'illegal' characters here from the query string.

		:param str query: The base query string which we are searching for
		:return str: Return the formatted URL
		"""
		allowed_characters: str = string.ascii_letters + string.digits + "_"

		query = "".join([char.lower() for char in query.replace(" ", "_") if char in allowed_characters])

		return "http://manganelo.com/search/story/" + query
